@@ -2,13 +2,10 @@ package phase
 
 import (
 	"fmt"
-	"strings"
 
 	"github.com/alessio/shellescape"
-	"github.com/k0sproject/dig"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
 	"github.com/k0sproject/rig/exec"
-	"gopkg.in/yaml.v2"
 	"k8s.io/client-go/tools/clientcmd"
 )
 
@@ -31,24 +28,6 @@ var readKubeconfig = func(h *cluster.Host) (string, error) {
 	return output, nil
 }
 
-var k0sConfig = func(h *cluster.Host) (dig.Mapping, error) {
-	cfgContent, err := h.Configurer.ReadFile(h, h.Configurer.K0sConfigPath())
-	if err != nil {
-		return nil, fmt.Errorf("read k0s config from host: %w", err)
-	}
-
-	var cfg dig.Mapping
-	if err := yaml.Unmarshal([]byte(cfgContent), &cfg); err != nil {
-		return nil, fmt.Errorf("unmarshal k0s config: %w", err)
-	}
-
-	if err != nil {
-		return nil, fmt.Errorf("parse k0s config: %w", err)
-	}
-
-	return cfg, nil
-}
-
 func (p *GetKubeconfig) DryRun() error {
 	p.DryMsg(p.Config.Spec.Hosts.Controllers()[0], "get admin kubeconfig")
 	return nil
@@ -58,34 +37,13 @@ func (p *GetKubeconfig) DryRun() error {
 func (p *GetKubeconfig) Run() error {
 	h := p.Config.Spec.Hosts.Controllers()[0]
 
-	cfg, err := k0sConfig(h)
-	if err != nil {
-		return err
-	}
-
 	output, err := readKubeconfig(h)
 	if err != nil {
 		return fmt.Errorf("read kubeconfig from host: %w", err)
 	}
 
 	if p.APIAddress == "" {
-		// the controller admin.conf is aways pointing to localhost, thus we need to change the address
-		// something usable from outside
-		address := h.Address()
-		if a, ok := cfg.Dig("spec", "api", "externalAddress").(string); ok && a != "" {
-			address = a
-		}
-
-		port := 6443
-		if p, ok := cfg.Dig("spec", "api", "port").(int); ok && p != 0 {
-			port = p
-		}
-
-		if strings.Contains(address, ":") {
-			p.APIAddress = fmt.Sprintf("https://[%s]:%d", address, port)
-		} else {
-			p.APIAddress = fmt.Sprintf("https://%s:%d", address, port)
-		}
+		p.APIAddress = p.Config.Spec.KubeAPIURL()
 	}
 
 	cfgString, err := kubeConfig(output, p.Config.Metadata.Name, p.APIAddress)
